# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimDate,
    HprimForeign,
)


class HeliosSegmentH(HeaderSegment):
    sender_id = HprimString(5)
    version = HprimString(13, default="LIS2-A2")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class HeliosSegmentP(Segment):
    patient_id = HprimString(4)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))

    orders = HprimForeign("O", "patient")


class HeliosSegmentO(Segment):
    specimen_id = HprimString(3)
    # un segment O par test
    test_id = HprimString((5, 4))
    dilution = HprimString((5, 5))
    priority = HprimString(6, default="R")

    results = HprimForeign("R", "order")


class HeliosSegmentR(Segment):
    test = HprimString((3, 4))
    dilution = HprimString((3, 5))
    value = HprimString((4, 1))
    aspect1 = HprimString((4, 2))
    aspect2 = HprimString((4, 3))
    aspect3 = HprimString((4, 4))
    status = HprimString(9)
    operator = HprimString(11)
    date_completed = HprimDate(13)
    instrument_id = HprimString(14)


class HeliosSegmentC(Segment):
    comment_source = HprimString(3)
    comment_code = HprimString((4, 1))
    comment_dilution = HprimString((4, 2))
    comment_text = HprimString((4, 3))
    comment_type = HprimString(5)


class HeliosSegmentQ(Segment):
    starting_range_id = HprimString((3, 2))
    universal_test_id = HprimString(5)


class HeliosSegmentL(Segment):
    terminaison_code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "helios",
        "label": "HELIOS AESKU INGEN"
    }
    SEGMENT_CLASS_MAP = {
        "H": HeliosSegmentH,
        "P": HeliosSegmentP,
        "O": HeliosSegmentO,
        "R": HeliosSegmentR,
        "C": HeliosSegmentC,
        "Q": HeliosSegmentQ,
        "L": HeliosSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"
    OPTIONS = [
        AstmMessageV2.Option(
            "helios_value_ignored",
            label="Ignorer les résultats ininterprétables ayant cette valeur dans le champs R.4.1"
        )
    ]

    def __init__(self, helios_value_ignored=None, **kwargs):
        self.value_ignored = helios_value_ignored
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = HeliosSegmentH(type_segment="H", **kwargs)
        message.end = HeliosSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        if worklist.context != "order":
            raise NotImplementedError("context %s" % (worklist.context,))
        msg = self.create(
            date=context["now"]
        )
        for patient in worklist.get_astm_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
        ):
            for sample, action in patient.samples:
                seg_p = msg.patients.add(
                    patient_id=sample.sample_id,
                    last_name=patient.last_name,
                    first_name=patient.first_name,
                )
                codes = []
                for order in sample.orders:
                    codes.append(order["code"])
                    seg_p.orders.add(
                        specimen_id=sample.sample_id,
                        test_id=order["code"],
                        dilution=order["dilution"] if order["dilution"] else ""
                    )
                for r in sample.results:  # Gestion des dilutions demandées
                    if 'DILUTION' in r.code:
                        for dil in r.value.split(';'):
                            code = "{},{}".format(r.code.split('_')[0], dil)
                            if code not in codes:
                                seg_p.orders.add(
                                    specimen_id=sample.sample_id,
                                    test_id=code,
                                )

        return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    type="patient",
                    sample_id=seg_q.starting_range_id
                )
        else:
            worklist = Worklist(context="result")
            for seg_p in message.patients:
                for seg_o in seg_p.orders:
                    for seg_r in seg_o.results:
                        comments = []
                        dilutions = []
                        bilans = []
                        superieur = False
                        if self.value_ignored and seg_r.value == self.value_ignored:
                            continue
                        for seg_c in seg_r.commentaires:
                            if seg_c.comment_code.startswith('Unspecified'):
                                t = seg_c.comment_code.split(':')
                                if len(t) == 2 and t[1].strip() == seg_r.dilution:
                                    superieur = True
                                if seg_c.comment_text:
                                    bilans = seg_c.comment_text.split('+')

                            # Ne traiter que les commentaires de type "T"
                            if seg_c.comment_type in ('C', 'I'):
                                continue
                            if seg_c.comment_code or seg_c.comment_dilution or seg_c.comment_text:
                                commentaires = "{}^{}^{}".format(
                                    seg_c.comment_code,
                                    seg_c.comment_dilution,
                                    seg_c.comment_text
                                )
                                comments.append(commentaires)
                            if seg_c.comment_code == 'IFA' and '[' in seg_c.comment_dilution and "dilutions" in seg_c.comment_dilution.lower():
                                for r in seg_c.comment_dilution.split('[')[1:]:
                                    r = r.replace(']', '')
                                    for d in r.split(';'):
                                        if '/' in d:
                                            dilutions.append(d.split('/')[1])

                        sample = worklist.get_or_create(
                            sample_id=seg_o.specimen_id,
                            type="patient",
                        )
                        if seg_r.value == '0':
                            titre = '<' + seg_r.dilution
                        elif superieur:
                            titre = '>' + seg_r.dilution
                        else:
                            titre = seg_r.dilution
                        # Demande de dilutions
                        if dilutions:
                            sample.results.add(
                                code="{}_DILUTION".format(seg_r.test, ),
                                value=';'.join(dilutions),
                                date=seg_r.date_completed,
                                dilution=seg_r.dilution,
                                operator=seg_r.operator,
                                instrument=seg_r.instrument_id,
                            )
                        # Demande de bilans
                        if bilans:
                            sample.results.add(
                                code="{}_BILAN".format(seg_r.test, ),
                                value=bilans,
                                date=seg_r.date_completed,
                                operator=seg_r.operator,
                                instrument=seg_r.instrument_id,
                            )
                        # Resultat par dilution
                        if seg_r.value.strip():
                            value = seg_r.value
                            if seg_r.aspect1:
                                value = "{}^{}".format(
                                    seg_r.value,
                                    seg_r.aspect1.strip()
                                )
                            if seg_r.aspect2:
                                value = "{}^{}^{}".format(
                                    seg_r.value,
                                    seg_r.aspect1.strip(),
                                    seg_r.aspect2.strip()
                                )
                            if seg_r.aspect3 and seg_r.aspect3.lower() not in ('positive', 'negative'):
                                value = "{}^{}^{}^{}".format(
                                    seg_r.value,
                                    seg_r.aspect1.strip(),
                                    seg_r.aspect2.strip(),
                                    seg_r.aspect3.strip()
                                )
                            sample.results.add(
                                code="{}_RESULT_{}".format(seg_r.test, seg_r.dilution),
                                value=value,
                                comments=comments,
                                date=seg_r.date_completed,
                                dilution=seg_r.dilution,
                                operator=seg_r.operator,
                                instrument=seg_r.instrument_id,
                            )
                            sample.results.add(
                                code="{}_RESULT".format(seg_r.test,),
                                value=value,
                                comments=comments,
                                date=seg_r.date_completed,
                                dilution=seg_r.dilution,
                                operator=seg_r.operator,
                                instrument=seg_r.instrument_id,
                            )
                            sample.results.add(
                                code="{}_TITRE".format(seg_r.test),
                                value=titre,
                                date=seg_r.date_completed,
                                operator=seg_r.operator,
                                instrument=seg_r.instrument_id,
                            )
                        # Aspect no 1
                        if seg_r.aspect1:
                            sample.results.add(
                                code="{}_ASPECT1".format(seg_r.test),
                                value=seg_r.aspect1,
                                date=seg_r.date_completed,
                                operator=seg_r.operator,
                                instrument=seg_r.instrument_id,
                            )
                        # Aspect no 2
                        if seg_r.aspect2:
                            sample.results.add(
                                code="{}_ASPECT2".format(seg_r.test),
                                value=seg_r.aspect2,
                                date=seg_r.date_completed,
                                operator=seg_r.operator,
                                instrument=seg_r.instrument_id,
                            )
                            sample.results.add(
                                code="{}_TITRE2".format(seg_r.test),
                                value=titre,
                                date=seg_r.date_completed,
                                operator=seg_r.operator,
                                instrument=seg_r.instrument_id,
                            )
                        # Aspect no 3
                        if seg_r.aspect3 and seg_r.aspect3.lower() not in ('positive', 'negative'):
                            sample.results.add(
                                code="{}_ASPECT3".format(seg_r.test),
                                value=seg_r.aspect3,
                                date=seg_r.date_completed,
                                operator=seg_r.operator,
                                instrument=seg_r.instrument_id,
                            )
                            sample.results.add(
                                code="{}_TITRE3".format(seg_r.test),
                                value=titre,
                                date=seg_r.date_completed,
                                operator=seg_r.operator,
                                instrument=seg_r.instrument_id,
                            )
                        # Les deux aspects
                        if seg_r.aspect1 or seg_r.aspect2:
                            sample.results.add(
                                code="{}_ASPECT".format(seg_r.test),
                                value="{}^{}".format(seg_r.aspect1, seg_r.aspect2),
                                date=seg_r.date_completed,
                                operator=seg_r.operator,
                                instrument=seg_r.instrument_id,
                            )
                        # test dedié aux aspect
                        for aspect in (seg_r.aspect1, seg_r.aspect2, seg_r.aspect3):
                            if aspect:
                                sample.results.add(
                                    code="{}_ASPECT_{}".format(seg_r.test, aspect),
                                    value=titre,
                                    flags=seg_r.value,  # 0 ou 1
                                    date=seg_r.date_completed,
                                    operator=seg_r.operator,
                                    instrument=seg_r.instrument_id,
                                )
        return worklist
